#ifndef WS2812_STM32F4_H
#define WS2812_STM32F4_H
/******************************************************
 *
 *	Driver for WS2812 led strips; Uses SPI port
 *
 *	(c) Peter Schulten (pitschu), Mlheim, Germany
 *	peter_(at)_pitschu.de
 *
 *	Die unvernderte Wiedergabe und Verteilung dieses gesamten Sourcecodes
 *	in beliebiger Form ist gestattet, sofern obiger Hinweis erhalten bleibt.
 *
 * 	Ich stelle diesen Sourcecode kostenlos zur Verfgung und biete daher weder
 *	Support an noch garantiere ich fr seine Funktionsfhigkeit. Auerdem
 *	bernehme ich keine Haftung fr die Folgen seiner Nutzung.

 *	Der Sourcecode darf nur zu privaten Zwecken verwendet und modifiziert werden.
 *	Darber hinaus gehende Verwendung bedarf meiner Zustimmung.
 *
 *	History
 *	Nov 2013	pitschu		Start of work
 *
*/

#include <math.h>
#include <stdint.h>

// pitschu: some very useful and often used macros
#define min(a,b) ((a)<(b)?(a):(b))
#define max(a,b) ((a)>(b)?(a):(b))
#define abs(x) ((x)>0?(x):-(x))
#define constrain(x,low,high) (((x)<(low))?(low):(((x)>(high))?(high):(x)))
#define round(x)     ((x)>=0?(long)((x)+0.5):(long)((x)-0.5))
#define putInRange(V,VMIN,VMAX) 		max(VMIN,min(VMAX,V))
#define mapToRange(V,VMIN0,VMAX0,VMIN1,VMAX1) ((VMIN1) + (putInRange(V,VMIN0,VMAX0) - (VMIN0)) * ((VMAX1) - (VMIN1)) / ((VMAX0) - (VMIN0)))

#define		LEDS_PHYS				15			// physical leds on stripe

#define	WS2812_DMA_CH				(UDMA_CH13_SSI2TX & 0x1F)		// = 13
#define WS2812_DMA_ASSIGN			UDMA_CH13_SSI2TX
#define	WS2812_SSI_BASE				SSI2_BASE
#define	WS2812_SSI_PERIPH			SYSCTL_PERIPH_SSI2
#define	WS2812_SSI_PORTPERIPH		SYSCTL_PERIPH_GPIOB
#define	WS2812_SSI_PORTPIN			GPIO_PIN_7
#define	WS2812_GPIO_SSI				GPIO_PB7_SSI2TX
#define	WS2812_INT_SSI				INT_SSI2
#define	WS2812_DMA_TRANSSIZE		1024				// must be lower or equal 1024 (DMA limit) !!

// number of null bytes to guarantee correct RESET pulse (> 50usec) for LEDs.
#define WS2812_RESET_LEN		(50 / 2)		// 50 bits with 1.25usec each is OK

// three colors per led, eight bits per color, 4 bytes per RGB byte (each SPI byte transfers 2 bits)
#define WS2812_DMABUF_LEN			(LEDS_PHYS * 3 * 4 + WS2812_RESET_LEN)

// two bit combinations to send via SPI (0-bit = short pulse (312ns), 1-bit = long pulse (625ns))
#define	WS_BITS_00				0x88
#define	WS_BITS_01				0x8c
#define	WS_BITS_10				0xc8
#define	WS_BITS_11				0xcc

typedef struct {
	uint8_t		R;
	uint8_t		G;
	uint8_t		B;
} rgbValue_t;

// ----------------------------- variables -----------------------------

extern rgbValue_t ws2812ledRGB[LEDS_PHYS];
extern rgbValue_t ws2812ledOVR[LEDS_PHYS];			// overlay color data for all leds (has higher prio than ws2812ledRGB
extern volatile unsigned char ws2812ledHasOVR[LEDS_PHYS];
extern volatile uint8_t	ledBusy;					// = 1 while dma is sending data to leds
extern volatile unsigned long ws2812ovrlayCounter;	// ignore overlay when 0 (decr in system ticker)
extern volatile uint16_t masterBrightness;			// between 0 and 100

extern volatile unsigned long ticker;

// ----------------------------- functions -----------------------------
void WS2812init(void);
int  WS2812update(void);	// ret 1 when DMA couldnt be started yet -> try again when LEDbusy is 0
int  WS2812startDMA(void);	// start the dma transfer (framebuffer to SPI)
void WS2812test(void);

#endif
